import { basename, dirname, extname, normalize } from "node:path";
import { MuseFileNotFoundError } from "#errors";

export interface LoadedFile {
  content: string;
  filePath: string;
  fileType: string;
  dirname: string;
  basename: string;
  mimeType: string;
}

export async function loadFileContent(filePath: string): Promise<LoadedFile> {
  const normalizedPath = normalize(filePath);
  const file = Bun.file(normalizedPath);

  const exists = await file.exists();
  if (!exists) {
    throw new MuseFileNotFoundError(normalizedPath);
  }

  const filecontent = await file.text();

  const extension = extname(normalizedPath);

  return {
    content: filecontent,
    filePath: normalizedPath,
    fileType: extension.slice(1),
    dirname: dirname(normalizedPath),
    basename: basename(normalizedPath, extension),
    mimeType: file.type,
  };
}
